import threading,utils

from models.project_data import ProjectModel, ProjectItemModel
from stores.mysql_store import MysqlStore
import data_collect,data_process,data_send


class SourceDataService:

    def __init__(self):
        self._store = MysqlStore()
        self._logger = utils.get_logger()

    def get_all_project_paginated(self, page: int, per_page: int, keyword: str = None, status: int = None) ->(list[ProjectModel], int):
        project_list, total_count = self._store.query_project_all_paginated(page, per_page, keyword, status)
        return project_list, total_count

    def get_all_project(self) ->list[ProjectModel]:
        project_list = self._store.query_project_all()
        return project_list

    def get_project_by_id(self, project_id:int) ->ProjectModel:
        project = self._store.query_project(project_id)
        return project

    def add_project(self, project:ProjectModel):
        self._store.insert_project(project)


    def update_project(self, project:ProjectModel):
        self._store.update_project(project)

    def start_task(self, project_id: int) -> (bool, str):
        data = self._store.query_project(project_id)
        if data:
            if data.status == 21 or  data.status == 31:
                return False, '正在采集数据中'
            if data.status == 22 or data.status == 32:
                return False, '正在分析处理中'
            if data.status == 32 or data.status == 33:
                return False, '正在上传数据中'
            thread = threading.Thread(target = self._process_and_send_project, args=(data,))
            thread.start()
            return True, ''
        else:
            return False, '项目不存在'

    def _process_and_send_project(self, project: ProjectModel) :
        # 启动分析处理
        if data_process.process_project(project):
            # 更新远程数据
            data_send.send_project_data(project)
            self._store.update_project_status(project.id,5)

    def process_project(self, project_id:int):
        data = self._store.query_project(project_id)
        if data:
            if data.status == 21 or data.status == 31:
                return False, '正在采集数据中'
            if data.status == 22 or data.status == 32:
                return False, '正在分析处理中'
            if data.status == 32 or data.status == 33:
                return False, '正在上传数据中'
            thread = threading.Thread(target=self._process_project, args=(data,))
            thread.start()
            return True, ''
        else:
            return False, '项目不存在'

    def _process_project(self, project: ProjectModel):
        if data_process.process_data(project):
            return True, ''
        else:
            self._logger.error(f"分析处理失败:{project.project_no}")
        return False, '分析处理失败'
    def start_send_data(self, project_id: int) -> (bool, str):
        data = self._store.query_project(project_id)
        if data:
            if data.status == 21 or data.status == 31:
                return False, '正在采集数据中'
            if data.status == 22 or data.status == 32:
                return False, '正在分析处理中'
            if data.status == 32 or data.status == 33:
                return False, '正在上传数据中'
            thread = threading.Thread(target=self._send_project_data, args=(data,))
            thread.start()


    def _send_project_data(self, project: ProjectModel):
        if data_send.send_project_data(project):
            self._store.update_project_status(project.id,5)
            return True, ''
        else:
            return False, '上传数据失败'

    def delete_project(self, project_id:int):
        project = self._store.query_project(project_id)
        if project:
            self._store.delete_project(project_id)
            return True,''
        else:
            return False, '项目不存在'

    def get_project_item_list_by_project_paginated(self, project_id: int, page: int, per_page: int, keyword: str = None) -> (list[ProjectItemModel], int):
        return self._store.query_project_items_by_project_paginated(project_id, page, per_page, keyword)
    def get_project_item_list_by_project_id(self, project_id:int) ->list[ProjectItemModel]:
        return  self._store.query_project_items_by_project(project_id)

    def get_project_item_by_id(self, item_id: int):
        return self._store.query_project_item_by_id(item_id)

    def add_project_item(self, item:ProjectItemModel):
        project_item = self._store.query_project_item(item.project_id, item.device_name, item.device_model)
        if project_item:
            return False
        else:
            project = self._store.query_project(item.project_id)
            project_item = ProjectItemModel(
                project_id=item.project_id,
                device_name=item.device_name,
                device_model=item.device_model,
                device_unit=item.device_unit,
                standard_version=project.standard_version,
                standard_no=item.standard_no,
            )
            self._store.insert_project_item(project_item)
            return True
    def update_project_item(self, item:ProjectItemModel):
        project_item = self._store.query_project_item_by_id(item.id)
        if project_item:
            project_item.device_name = item.device_name
            project_item.device_model = item.device_model
            project_item.device_unit = item.device_unit
            project_item.device_count = item.device_count
            project_item.standard_no = item.standard_no
            self._store.update_project_data(project_item)
            #TODO 更新标准数据库

            #TODO 更新远程数据

            return True
        else:
            return False
    def delete_project_item(self, item_id:int):
        project_item = self._store.query_project_item_by_id(item_id)
        if project_item:
            self._store.delete_project_item_by_id(project_item.id)
            return True
        else:
            return False
